import { randomIntInRange } from '../utils/Random';

interface BuilderMemory extends CreepMemory {
  building: boolean | undefined;
  sourceNum: number | undefined;
}

export function runAsBuilder(creep: Creep) {
  const memory = creep.memory as BuilderMemory;

  if (memory.building && creep.store[RESOURCE_ENERGY] === 0) {
    memory.building = false;
  }

  if (!memory.building && creep.store.getFreeCapacity() === 0) {
    memory.building = true;
  }

  if (memory.sourceNum === undefined) {
    const sources = creep.room.find(FIND_SOURCES);
    memory.sourceNum = randomIntInRange(0, sources.length);
  }

  if (memory.building) {
    if (build(creep)) {
      return;
    }
    if (repair(creep)) {
      return;
    }
    const spawns = creep.room.find(FIND_MY_SPAWNS);
    if (spawns.length > 0) {
      creep.say('To spawn');
      creep.moveTo(spawns[0]);
    }
  } else {
    const sources = creep.room.find(FIND_SOURCES);
    const sourceNum = memory.sourceNum;
    if (creep.harvest(sources[sourceNum]) == ERR_NOT_IN_RANGE) {
      creep.say('🔄 harvest ' + sourceNum);
      creep.moveTo(sources[sourceNum], { visualizePathStyle: { stroke: '#ffaa00' } });
    }
  }
}

function build(creep: Creep): boolean {
  const targets = creep.room.find(FIND_MY_CONSTRUCTION_SITES);
  if (targets.length === 0) {
    return false;
  }
  if (creep.build(targets[0]) == ERR_NOT_IN_RANGE) {
    creep.say('🚧 build');
    creep.moveTo(targets[0], { visualizePathStyle: { stroke: '#ffffff' } });
  }
  return true;
}

function repair(creep: Creep) {
  const targets = creep.room.find(FIND_STRUCTURES).filter((t) => t.hits < t.hitsMax);
  if (targets.length === 0) {
    return false;
  }
  if (creep.repair(targets[0]) == ERR_NOT_IN_RANGE) {
    creep.say('🚧 repair');
    creep.moveTo(targets[0], { visualizePathStyle: { stroke: '#ffffff' } });
  }
  return true;
}
